import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class MedianPanel extends JPanel implements Runnable {
    public MedianPanel() {
	setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

	JPanel samplePanel = new JPanel();
	samplePanel.setAlignmentX(Component.CENTER_ALIGNMENT);
        add(samplePanel);
	samplePanel.setLayout(new GridLayout(2,5,5,5));
	samplePanel.setBorder(
	    BorderFactory.createCompoundBorder(
	        BorderFactory.createTitledBorder("Doodling Area"),
	        BorderFactory.createEmptyBorder(0,5,5,5)));
	for (int i = 0; i < 10; i++) {
	    canvas[i] = new DoodleCanvas();
            samplePanel.add(canvas[i]);
	}

	add(Box.createRigidArea(new Dimension(0,5)));
        JLabel l = new JLabel("Draw 10 times the same character (or not), and click on \"Update Medians\".");
        l.setFont(UIManager.getFont("TextPane.font"));
	l.setAlignmentX(Component.CENTER_ALIGNMENT);
        add(l);
	add(Box.createRigidArea(new Dimension(0,5)));

	JPanel medianPanel = new JPanel();
	medianPanel.setAlignmentX(Component.CENTER_ALIGNMENT);
        add(medianPanel);
	medianPanel.setLayout(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();

	c.gridx = 0;
	c.gridy = 0;
	medianPanel.add(new JLabel("Generalized Median"), c);
	c.gridy = 1;
        genMedianCanvas = new DoodleCanvas();
	genMedianCanvas.setEnabled(false);
        medianPanel.add(genMedianCanvas, c);
	c.gridy = 2;
	genMedianSODLabel = new JLabel("SOD = ");
	medianPanel.add(genMedianSODLabel, c);
	c.gridy = 3;
	genMedianTimeLabel = new JLabel("time = ");
	medianPanel.add(genMedianTimeLabel, c);

	c.insets = new Insets(0,10,0,0);
	c.gridx = 1;
	c.gridy = 0;
	medianPanel.add(new JLabel("Set Median"), c);
	c.gridy = 1;
        setMedianCanvas = new DoodleCanvas();
	setMedianCanvas.setEnabled(false);
        medianPanel.add(setMedianCanvas, c);
	c.gridy = 2;
	setMedianSODLabel = new JLabel("SOD = ");
	medianPanel.add(setMedianSODLabel, c);
	c.gridy = 3;
	setMedianTimeLabel = new JLabel("time = ");
	medianPanel.add(setMedianTimeLabel, c);

	c.gridx = 0;
	c.gridy = 4;
	c.gridwidth = 3;
	goButton = new JButton("Update Medians");
	medianPanel.add(goButton,c);
	goButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
	        // reset all canvases to initial state and disable them
	        for (int i = 0; i < canvas.length; i++) {
	            canvas[i].setBackground(canvas[i].getParent().getBackground());
		    canvas[i].setEnabled(false);
	        }

                genMedianCanvas.setBackground(color1);
                genMedianCanvas.setString(null);
	        genMedianSODLabel.setText("SOD = ");
	        genMedianTimeLabel.setText("time = ");

                setMedianCanvas.setBackground(setMedianCanvas.getParent().getBackground());
	        setMedianCanvas.setString(null);
                setMedianSODLabel.setText("SOD = ");
	        setMedianTimeLabel.setText("time = ");

		// disable button and start the worker thread
		goButton.setEnabled(false);
	        new Thread(MedianPanel.this).start();
	    }
	});
    }
    
    // this code is run as part of the worker thread when the goButton is clicked
    public void run() {
        // the string set to be filled with strings from canvases
	final StringSet2D ss = new StringSet2D();

        // run stuff for dynamic computation of the generalized median 
	final long startTime = System.currentTimeMillis();
	for (int i = 0; i < canvas.length; i++) {
	    String2D s = canvas[i].getString();
	    if (s != null && s.size() > 0) {
		ss.add(s);
		final DoodleCanvas canvasToUpdate = canvas[i];
		final int currentSOD = ss.getSOD(ss.genMedian); // will use up some time...
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
		        canvasToUpdate.setBackground(color1);
                        genMedianCanvas.setString(ss.genMedian);
	                genMedianSODLabel.setText("SOD = " + currentSOD);
	                genMedianTimeLabel.setText("time = " + (System.currentTimeMillis() - startTime)/1000.0 + " s");
                    }
                });
	    }
	}

        // update stuff for set median
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                genMedianCanvas.setBackground(genMedianCanvas.getParent().getBackground());
                setMedianCanvas.setBackground(color2);

		long startTime = System.currentTimeMillis();
		setMedianCanvas.setString(ss.getSetMedian());
		long endTime = System.currentTimeMillis();
		setMedianSODLabel.setText("SOD = " + ss.getSOD(ss.getSetMedian()));
		setMedianTimeLabel.setText("time = " + (endTime - startTime)/1000.0 + " s");

		for (int i = 0; i < canvas.length; i++) {
		    if (ss.getSetMedian() != null && canvas[i].getString() == ss.getSetMedian())
			canvas[i].setBackground(color2);
		    else
			canvas[i].setBackground(canvas[i].getParent().getBackground());

		    canvas[i].setEnabled(true);
		}

		goButton.setEnabled(true);
            }
        });

    /*

	Sequence2D S = canvas[0].getString().distance(canvas[1].getString());
	System.out.println(S.cost());
	Sequence2D m = S.getSubSequence(0.5);
	canvas[2].setString((String2D)canvas[0].getString().clone());
	canvas[2].getString().apply(m);
	canvas[2].repaint();

	System.out.println(canvas[0].getString().distance(canvas[2].getString()).cost());
	System.out.println(canvas[1].getString().distance(canvas[2].getString()).cost());


    //		System.out.println(S);
    */
    }

    private static final Color color1 = Color.decode("0xB8CFE5");
    private static final Color color2 = Color.decode("0xadffad");

    DoodleCanvas[] canvas = new DoodleCanvas[10];
    DoodleCanvas setMedianCanvas, genMedianCanvas;
    JLabel setMedianSODLabel,  genMedianSODLabel, 
           setMedianTimeLabel, genMedianTimeLabel;
    JButton goButton;
}
