import java.util.*;

class StringSet2D extends ArrayList<String2D> {
    private static final double alphaStep = 0.1;
    private static final int randomRestarts = 15;
    
    public String2D setMedian = null, genMedian = null;

    public boolean add(String2D x) {
	if(!super.add(x))
	    return false;

	// if no median, set to x
	if (genMedian == null) {
	    genMedian = x;
	    return true;
	}

	// otherwise, update generalized median with a sequence providing 
	// minimal Sum Of Distances
        Sequence2D S = genMedian.distance(x);
	String2D newMedian = null;
	int minSOD = Integer.MAX_VALUE;
	for (double alpha = 0; alpha <= 1.0; alpha += alphaStep) {
	    for (int i = 0; i < randomRestarts; i++) {
		// compute median at alpha
		String2D tempMedian = (String2D)genMedian.clone();
		tempMedian.apply(S.getSubSequence(alpha));

		// get SOD from candidate median to all strings in set
        	int SOD = getSOD(tempMedian);

		// take the median with the smallest SOD, and favor
		// medians at alpha = 0.5 if equal SOD 
	        if (SOD < minSOD || (SOD == minSOD && alpha <= 0.5)) {
		    minSOD = SOD;
	            newMedian = tempMedian;
		}
	    }
	}
	genMedian = newMedian;
	
	setMedian = null; // we don't compute it right now...

	return true;
    }

    public String2D getSetMedian() {
        if (setMedian == null) {
	    // find the set median...
	    int minSOD = Integer.MAX_VALUE;
	    Iterator<String2D> it = iterator();
	    while (it.hasNext()) {
		String2D newMedian = it.next();
	        int SOD = getSOD(newMedian);
		if (SOD < minSOD) {
	            minSOD = SOD;
	            setMedian = newMedian;
		}
	    }
	}
	return setMedian;
    }
    
    public int getSOD(String2D s) {
        int SOD = 0;
	Iterator<String2D> it = iterator();
	while (it.hasNext()) {
	    SOD += s.distance(it.next()).cost();
 	}
	return SOD;
    }

}
